/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_VED4DPC_H
#define DPC_IMPL_TYPE_VED4DPC_H

#include <cfloat>
#include <cmath>
#include "impl_type_float.h"
#include "impl_type_uint8.h"
#include "impl_type_vectoruint8.h"
#include "impl_type_vectorfloat.h"

namespace dpc {
struct VED4DPC {
    ::Float LongitudinalVelocity{ 0.0F };
    ::Float LongitudinalAcceleration{ 0.0F };
    ::Float LateralVelocity{ 0.0F };
    ::Float LateralAcceleration{ 0.0F };
    ::UInt8 MotionState{ 0U };
    ::Float Yawrate{ 0.0F };
    ::Float SteeringAngle{ 0.0F };
    ::Float SteeringRate{ 0.0F };
    ::VectorUInt8 GeneralCanUnit8SignalsPassed;
    ::VectorFloat GeneralCanFloatSignalsPassed;

    static bool IsPlane()
    {
        return false;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(LongitudinalVelocity);
        fun(LongitudinalAcceleration);
        fun(LateralVelocity);
        fun(LateralAcceleration);
        fun(MotionState);
        fun(Yawrate);
        fun(SteeringAngle);
        fun(SteeringRate);
        fun(GeneralCanUnit8SignalsPassed);
        fun(GeneralCanFloatSignalsPassed);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(LongitudinalVelocity);
        fun(LongitudinalAcceleration);
        fun(LateralVelocity);
        fun(LateralAcceleration);
        fun(MotionState);
        fun(Yawrate);
        fun(SteeringAngle);
        fun(SteeringRate);
        fun(GeneralCanUnit8SignalsPassed);
        fun(GeneralCanFloatSignalsPassed);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("LongitudinalVelocity", LongitudinalVelocity);
        fun("LongitudinalAcceleration", LongitudinalAcceleration);
        fun("LateralVelocity", LateralVelocity);
        fun("LateralAcceleration", LateralAcceleration);
        fun("MotionState", MotionState);
        fun("Yawrate", Yawrate);
        fun("SteeringAngle", SteeringAngle);
        fun("SteeringRate", SteeringRate);
        fun("GeneralCanUnit8SignalsPassed", GeneralCanUnit8SignalsPassed);
        fun("GeneralCanFloatSignalsPassed", GeneralCanFloatSignalsPassed);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("LongitudinalVelocity", LongitudinalVelocity);
        fun("LongitudinalAcceleration", LongitudinalAcceleration);
        fun("LateralVelocity", LateralVelocity);
        fun("LateralAcceleration", LateralAcceleration);
        fun("MotionState", MotionState);
        fun("Yawrate", Yawrate);
        fun("SteeringAngle", SteeringAngle);
        fun("SteeringRate", SteeringRate);
        fun("GeneralCanUnit8SignalsPassed", GeneralCanUnit8SignalsPassed);
        fun("GeneralCanFloatSignalsPassed", GeneralCanFloatSignalsPassed);
    }

    friend bool operator==(const ::dpc::VED4DPC& lhs, const ::dpc::VED4DPC& rhs) noexcept
    {
        return (fabs(static_cast<double>(lhs.LongitudinalVelocity - rhs.LongitudinalVelocity)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.LongitudinalAcceleration - rhs.LongitudinalAcceleration)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.LateralVelocity - rhs.LateralVelocity)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.LateralAcceleration - rhs.LateralAcceleration)) < DBL_EPSILON) && (lhs.MotionState == rhs.MotionState) && (fabs(static_cast<double>(lhs.Yawrate - rhs.Yawrate)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.SteeringAngle - rhs.SteeringAngle)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.SteeringRate - rhs.SteeringRate)) < DBL_EPSILON) && (lhs.GeneralCanUnit8SignalsPassed == rhs.GeneralCanUnit8SignalsPassed) && (lhs.GeneralCanFloatSignalsPassed == rhs.GeneralCanFloatSignalsPassed);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_VED4DPC_H
